#include "Board.h"
#include "dbgu.h"

typedef void (*funct)(void);

//* Internal functions

void Wait (unsigned long Speed);
void Init_LED (void);
void Set_LED (unsigned int Led);
void Clean_LED (unsigned int Led);
void Trace_Toggle_LED( unsigned int led) ;

//* external function
extern unsigned disableIRQ();
extern unsigned long _bootsize;
extern unsigned int _test;

#define AT91B_MASTER_CLOCK	((18432000*73/14)/2)   // Output PLL Clock (48 MHz)
volatile unsigned long count = 0;

//*--------------------------------------------------------------------------------------
//* Function Name       : Wait
//* Object              : Software waiting loop
//* Input Parameters    : Speed
//* Output Parameters   : none
//*--------------------------------------------------------------------------------------
void Wait (unsigned long Speed)
{
    unsigned long waiting_time;
    
    for(waiting_time = 0; waiting_time < Speed; waiting_time++);
}

//*----------------------------------------------------------------------------
//* \fn    Init_Ports
//* \brief This function Initializs the LEDs and other ports
//*----------------------------------------------------------------------------
void Init_Ports (void)
{
    // Led init
    //Initializes only LED1-LED3
    AT91PS_PIO pLed = AT91C_BASE_PIOA;
    // First, enable the clock of the PIOA
    AT91F_PMC_EnablePeriphClock ( AT91C_BASE_PMC, 1 << AT91C_ID_PIOA ) ;
    /* Configure the pin in output */
    pLed->PIO_OER = (LED1 | LED2 | LED3 | AT91C_PIO_PA21);
    /* Set the PIO controller in PIO mode instead of peripheral mode */
    pLed->PIO_PER = (LED1 | LED2 | LED3 | AT91C_PIO_PA21);
    /* Disable pull-up */
    pLed->PIO_PPUDR = (LED1 | LED2 | LED3 | AT91C_PIO_PA21);
  
    /* Set the default state for the led */
    pLed->PIO_SODR = LED1 | LED2 | LED3;
}

//*----------------------------------------------------------------------------
//* Function Name       : Trace_Toggel_LED
//* Object              : Toggel a LED
//*----------------------------------------------------------------------------
void Trace_Toggle_LED (unsigned int Led)
{
    if ( (AT91F_PIO_GetInput(AT91C_BASE_PIOA) & Led ) == Led )
    {
        AT91F_PIO_ClearOutput( AT91C_BASE_PIOA, Led );
    }
    else
    {
        AT91F_PIO_SetOutput( AT91C_BASE_PIOA, Led );
    }
}

//*----------------------------------------------------------------------------
//* Function Name       : Set_LED
//* Object              : Set a LED
//*----------------------------------------------------------------------------
void Set_LED (unsigned int Led)
{
    AT91F_PIO_SetOutput( AT91C_BASE_PIOA, Led );
}

//*----------------------------------------------------------------------------
//* Function Name       : Clear_LED
//* Object              : Clear a LED
//*----------------------------------------------------------------------------
void Clear_LED (unsigned int Led)
{
    AT91F_PIO_ClearOutput( AT91C_BASE_PIOA, Led );
}

//*--------------------------------------------------------------------------------------
//* Function Name       : main
//* Object              :
//*--------------------------------------------------------------------------------------
int main ( void )
{
    int i;

    //configure_tc();
    AT91F_DBGU_Init();
    AT91F_DBGU_Printk("Bootloader running...\n\r");

    AT91C_BASE_AIC->AIC_IDCR = ~0;
    AT91C_BASE_AIC->AIC_ICCR = ~0;

    for (i = 0; i < 8; ++i) {
        AT91C_BASE_AIC->AIC_EOICR = 0;           /* write AIC_EOICR 8 times */
    }


    //Run the application
    ((funct)0x00104000)();

}
